@@ -1,5 +1,5 @@
 /* xSYM symbol-file support for BFD.
-   Copyright (C) 1999-2014 Free Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -26,38 +26,42 @@
 #include "bfd.h"
 #include "libbfd.h"
 
-#define bfd_sym_close_and_cleanup                   _bfd_generic_close_and_cleanup
-#define bfd_sym_bfd_free_cached_info                _bfd_generic_bfd_free_cached_info
-#define bfd_sym_new_section_hook                    _bfd_generic_new_section_hook
-#define bfd_sym_bfd_is_local_label_name             bfd_generic_is_local_label_name
-#define bfd_sym_bfd_is_target_special_symbol       ((bfd_boolean (*) (bfd *, asymbol *)) bfd_false)
-#define bfd_sym_get_lineno                          _bfd_nosymbols_get_lineno
-#define bfd_sym_find_nearest_line                   _bfd_nosymbols_find_nearest_line
-#define bfd_sym_find_inliner_info                   _bfd_nosymbols_find_inliner_info
-#define bfd_sym_bfd_make_debug_symbol               _bfd_nosymbols_bfd_make_debug_symbol
-#define bfd_sym_read_minisymbols                    _bfd_generic_read_minisymbols
-#define bfd_sym_minisymbol_to_symbol                _bfd_generic_minisymbol_to_symbol
-#define bfd_sym_set_arch_mach                       _bfd_generic_set_arch_mach
-#define bfd_sym_get_section_contents                _bfd_generic_get_section_contents
-#define bfd_sym_set_section_contents                _bfd_generic_set_section_contents
+#define bfd_sym_close_and_cleanup		    _bfd_generic_close_and_cleanup
+#define bfd_sym_bfd_free_cached_info		    _bfd_generic_bfd_free_cached_info
+#define bfd_sym_new_section_hook		    _bfd_generic_new_section_hook
+#define bfd_sym_bfd_is_local_label_name		    bfd_generic_is_local_label_name
+#define bfd_sym_bfd_is_target_special_symbol	    _bfd_bool_bfd_asymbol_false
+#define bfd_sym_get_lineno			    _bfd_nosymbols_get_lineno
+#define bfd_sym_find_nearest_line		    _bfd_nosymbols_find_nearest_line
+#define bfd_sym_find_line			    _bfd_nosymbols_find_line
+#define bfd_sym_find_inliner_info		    _bfd_nosymbols_find_inliner_info
+#define bfd_sym_get_symbol_version_string	    _bfd_nosymbols_get_symbol_version_string
+#define bfd_sym_bfd_make_debug_symbol		    _bfd_nosymbols_bfd_make_debug_symbol
+#define bfd_sym_read_minisymbols		    _bfd_generic_read_minisymbols
+#define bfd_sym_minisymbol_to_symbol		    _bfd_generic_minisymbol_to_symbol
+#define bfd_sym_set_arch_mach			    _bfd_generic_set_arch_mach
+#define bfd_sym_get_section_contents		    _bfd_generic_get_section_contents
+#define bfd_sym_set_section_contents		    _bfd_generic_set_section_contents
 #define bfd_sym_bfd_get_relocated_section_contents  bfd_generic_get_relocated_section_contents
-#define bfd_sym_bfd_relax_section                   bfd_generic_relax_section
-#define bfd_sym_bfd_gc_sections                     bfd_generic_gc_sections
-#define bfd_sym_bfd_lookup_section_flags            bfd_generic_lookup_section_flags
-#define bfd_sym_bfd_merge_sections                  bfd_generic_merge_sections
-#define bfd_sym_bfd_is_group_section                bfd_generic_is_group_section
-#define bfd_sym_bfd_discard_group                   bfd_generic_discard_group
-#define bfd_sym_section_already_linked              _bfd_generic_section_already_linked
-#define bfd_sym_bfd_define_common_symbol            bfd_generic_define_common_symbol
-#define bfd_sym_bfd_link_hash_table_create          _bfd_generic_link_hash_table_create
-#define bfd_sym_bfd_link_hash_table_free            _bfd_generic_link_hash_table_free
-#define bfd_sym_bfd_link_add_symbols                _bfd_generic_link_add_symbols
-#define bfd_sym_bfd_link_just_syms                  _bfd_generic_link_just_syms
+#define bfd_sym_bfd_relax_section		    bfd_generic_relax_section
+#define bfd_sym_bfd_gc_sections			    bfd_generic_gc_sections
+#define bfd_sym_bfd_lookup_section_flags	    bfd_generic_lookup_section_flags
+#define bfd_sym_bfd_merge_sections		    bfd_generic_merge_sections
+#define bfd_sym_bfd_is_group_section		    bfd_generic_is_group_section
+#define bfd_sym_bfd_discard_group		    bfd_generic_discard_group
+#define bfd_sym_section_already_linked		    _bfd_generic_section_already_linked
+#define bfd_sym_bfd_define_common_symbol	    bfd_generic_define_common_symbol
+#define bfd_sym_bfd_link_hide_symbol		    _bfd_generic_link_hide_symbol
+#define bfd_sym_bfd_define_start_stop		    bfd_generic_define_start_stop
+#define bfd_sym_bfd_link_hash_table_create	    _bfd_generic_link_hash_table_create
+#define bfd_sym_bfd_link_add_symbols		    _bfd_generic_link_add_symbols
+#define bfd_sym_bfd_link_just_syms		    _bfd_generic_link_just_syms
 #define bfd_sym_bfd_copy_link_hash_symbol_type \
   _bfd_generic_copy_link_hash_symbol_type
-#define bfd_sym_bfd_final_link                      _bfd_generic_final_link
-#define bfd_sym_bfd_link_split_section              _bfd_generic_link_split_section
-#define bfd_sym_get_section_contents_in_window      _bfd_generic_get_section_contents_in_window
+#define bfd_sym_bfd_final_link			    _bfd_generic_final_link
+#define bfd_sym_bfd_link_split_section		    _bfd_generic_link_split_section
+#define bfd_sym_get_section_contents_in_window	    _bfd_generic_get_section_contents_in_window
+#define bfd_sym_bfd_link_check_relocs		    _bfd_generic_link_check_relocs
 
 extern const bfd_target sym_vec;
 
@@ -1803,7 +1807,7 @@ bfd_sym_print_type_information_table_entry (bfd *abfd,
 
   fprintf (f, "\n            ");
 
-  buf = alloca (entry->physical_size);
+  buf = malloc (entry->physical_size);
   if (buf == NULL)
     {
       fprintf (f, "[ERROR]\n");
@@ -1812,11 +1816,13 @@ bfd_sym_print_type_information_table_entry (bfd *abfd,
   if (bfd_seek (abfd, entry->offset, SEEK_SET) < 0)
     {
       fprintf (f, "[ERROR]\n");
+      free (buf);
       return;
     }
   if (bfd_bread (buf, entry->physical_size, abfd) != entry->physical_size)
     {
       fprintf (f, "[ERROR]\n");
+      free (buf);
       return;
     }
 
@@ -1836,6 +1842,7 @@ bfd_sym_print_type_information_table_entry (bfd *abfd,
 
   if (offset != entry->physical_size)
     fprintf (f, "\n            [parser used %lu bytes instead of %lu]", offset, entry->physical_size);
+  free (buf);
 }
 
 void
@@ -2326,16 +2333,16 @@ const bfd_target sym_vec =
     _bfd_dummy_target,
   },
   {				/* bfd_set_format.  */
-    bfd_false,
+    _bfd_bool_bfd_false_error,
     bfd_sym_mkobject,
-    bfd_false,
-    bfd_false,
+    _bfd_bool_bfd_false_error,
+    _bfd_bool_bfd_false_error,
   },
   {				/* bfd_write_contents.  */
-    bfd_false,
-    bfd_true,
-    bfd_false,
-    bfd_false,
+    _bfd_bool_bfd_false_error,
+    _bfd_bool_bfd_true,
+    _bfd_bool_bfd_false_error,
+    _bfd_bool_bfd_false_error,
   },
 
   BFD_JUMP_TABLE_GENERIC (bfd_sym),
