@@ -1,6 +1,6 @@
 /* GNU/Linux on ARM target support, prototypes.
 
-   Copyright (C) 2006-2018 Free Software Foundation, Inc.
+   Copyright (C) 2006-2019 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,6 +17,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+#ifndef ARM_LINUX_TDEP_H
+#define ARM_LINUX_TDEP_H
+
 struct regset;
 struct regcache;
 
@@ -64,3 +67,4 @@ void arm_linux_collect_nwfpe (const struct regset *regset,
 #define HWCAP_VFPv3     8192
 #define HWCAP_VFPv3D16  16384
 
+#endif /* ARM_LINUX_TDEP_H */
