@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2007 Free Software Foundation, Inc.
+   Copyright 2007-2019 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,5 +20,5 @@
 int
 main()
 {
-  return 0;
+  return integer;
 }
