@@ -1,6 +1,5 @@
 /* Types for Cpu tools GENerated simulators.
-   Copyright (C) 1996, 1997, 1998, 1999, 2007, 2008, 2009
-   Free Software Foundation, Inc.
+   Copyright (C) 1996-2019 Free Software Foundation, Inc.
    Contributed by Cygnus Support.
 
 This file is part of GDB, the GNU debugger.
@@ -27,10 +26,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 /* Miscellaneous cgen configury defined here as this file gets
    included soon enough.  */
 
-/* Indicate we support --profile-model.  */
-#undef SIM_HAVE_MODEL
-#define SIM_HAVE_MODEL
-
 /* Indicate we support --{profile,trace}-{range,function}.  */
 #undef SIM_HAVE_ADDR_RANGE
 #define SIM_HAVE_ADDR_RANGE
@@ -61,7 +56,7 @@ typedef enum mode_type {
 #define MAX_TARGET_MODES ((int) MODE_TARGET_MAX)
 #define MAX_MODES ((int) MODE_MAX)
 
-extern const char *mode_names[];
+extern const char * const mode_names[];
 #define MODE_NAME(m) (mode_names[m])
 
 typedef void VOID;
@@ -80,7 +75,6 @@ typedef unsigned64 UDI;
 #define GETHIDI(di) ((SI) ((UDI) (di) >> 32))
 #define SETLODI(di, val) ((di) = (((di) & 0xffffffff00000000LL) | (val)))
 #define SETHIDI(di, val) ((di) = (((di) & 0xffffffffLL) | (((DI) (val)) << 32)))
-#define SETDI(di, hi, lo) ((di) = MAKEDI (hi, lo))
 #define MAKEDI(hi, lo) ((((DI) (SI) (hi)) << 32) | ((UDI) (USI) (lo)))
 #else
 /* DI mode support if "long long" doesn't exist.
@@ -94,7 +88,6 @@ typedef DI UDI;
 #define GETHIDI(di) ((di).hi)
 #define SETLODI(di, val) ((di).lo = (val))
 #define SETHIDI(di, val) ((di).hi = (val))
-#define SETDI(di, hi, lo) ((di) = MAKEDI (hi, lo))
 extern DI make_struct_di (SI, SI);
 #define MAKEDI(hi, lo) (make_struct_di ((hi), (lo)))
 #endif
