@@ -3,7 +3,7 @@
 #cc: additional_flags=-pthread
 #xerror:
 #output: Invalid sigreturn syscall: no signal handler active (0x1, 0x2, 0x3, 0x4, 0x5, 0x6)\n
-#output: program stopped with signal 4.\n
+#output: program stopped with signal 4 (*).\n
 */
 
 #include <stdlib.h>
@@ -14,6 +14,7 @@
 #include <sys/types.h>
 #include <sys/syscall.h>
 #include <signal.h>
+#include <errno.h>
 
 static void *
 process (void *arg)
@@ -27,7 +28,11 @@ int main (void)
 {
   pthread_t th_a;
   if (pthread_create (&th_a, NULL, process, (void *) "a") == 0)
-    syscall (SYS_sigreturn, 1, 2, 3, 4, 5, 6);
+    {
+      int err = syscall (SYS_sigreturn, 1, 2, 3, 4, 5, 6);
+      if (err == -1 && errno == ENOSYS)
+	printf ("ENOSYS\n");
+    }
   printf ("xyzzy\n");
   exit (0);
 }
